#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>
#include "AkelDLL.h"


//Defines
#ifndef IMF_AUTOFONT
  #define IMF_AUTOFONT 0x0002
#endif
#ifndef IMF_AUTOFONTSIZEADJUST
  #define IMF_AUTOFONTSIZEADJUST 0x0010
#endif

//Functions prototypes
LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
HWND hMainWnd;
BOOL bOldWindows;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
WNDPROCDATA *NewMainProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(-1, -1, -1, -1);
  pv->pPluginName="PlainText";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == AKDN_EDIT_ONSTART)
  {
    EDITINFO ei;
    int nLangOptions;
    int nTextMode;

    if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
    {
      nLangOptions=SendMessage((HWND)wParam, EM_GETLANGOPTIONS, 0, 0);
      nLangOptions|=IMF_AUTOFONT;
      nLangOptions|=IMF_AUTOFONTSIZEADJUST;
      SendMessage((HWND)wParam, EM_SETLANGOPTIONS, 0, nLangOptions);

      nTextMode=SendMessage((HWND)wParam, EM_GETTEXTMODE, 0, 0);
      nTextMode&=~TM_RICHTEXT;
      nTextMode|=TM_PLAINTEXT;
      SendMessage((HWND)wParam, EM_SETTEXTMODE, nTextMode, 0);

      SendMessage(hMainWnd, AKD_SETMODIFY, (WPARAM)ei.hWndEdit, ei.bModified);
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hMainWnd=pd->hMainWnd;
  bOldWindows=pd->bOldWindows;
}

void InitMain()
{
  bInitMain=TRUE;

  //SubClass
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&NewMainProcData);
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
